@@ -2,22 +2,6 @@ import numpy as np
 from .misc import detrend
 fft = np.fft.fft
 
-## class FFTobj(np.ndarray):
-
-##     @property
-##     def Spec(self,):
-##         return np.abs(self) ** 2
-
-##     def __new__(cls, input_array, info=None):
-##         obj = np.asarray(input_array).view(cls)
-##         obj.info = info
-##         return obj
-
-##     def __array_finalize__(self, obj):
-##         if obj is None:
-##             return
-##         self.info = getattr(obj, 'info', None)
-
 
 def psd_freq(nfft, fs, full=False):
     """
@@ -25,10 +9,8 @@ def psd_freq(nfft, fs, full=False):
 
     Parameters
     ----------
-
     fs : float
       The sampling frequency (e.g. samples/sec)
-
     nfft : int
       The number of samples in a window.
     full : bool (default: False)
@@ -36,9 +18,8 @@ def psd_freq(nfft, fs, full=False):
 
     Returns
     -------
-
-    freq : ndarray
-      The frequency vector.
+    freq : |np.ndarray|
+      The frequency vector, in same units as 'fs'
 
     """
     fs = np.float64(fs)
@@ -52,12 +33,14 @@ def psd_freq(nfft, fs, full=False):
 def _getwindow(window, nfft):
     if window == 'hann':
         window = np.hanning(nfft)
+    elif window == 'hamm':
+        window = np.hamming(nfft)
     elif window is None or window == 1:
         window = np.ones(nfft)
     return window
 
 
-def _stepsize(l, nfft, nens=None, step=None):
+def stepsize(l, nfft, nens=None, step=None):
     """
     Calculates the fft-step size for a length *l* array.
 
@@ -77,6 +60,7 @@ def _stepsize(l, nfft, nens=None, step=None):
     step    : The step size.
     nens    : The number of ensemble ffts to average together.
     nfft    : The number of points in the fft (set to l if nfft>l).
+
     """
     if l < nfft:
         nfft = l
@@ -93,28 +77,22 @@ def _stepsize(l, nfft, nens=None, step=None):
         return int((l - nfft) / (nens - 1)), int(nens), int(nfft)
 
 
-def cohere(a, b, nfft, window='hann', debias=True, noise=(0, 0)):
-    r"""
+def coherence(a, b, nfft, window='hann', debias=True, noise=(0, 0)):
+    """
     Computes the magnitude-squared coherence of `a` and `b`.
 
     Parameters
     ----------
-
-    a : np.ndarray
+    a : |np.ndarray|
       The first array over which to compute coherence.
-
-    b : np.ndarray
+    b : |np.ndarray|
       The second array over which to compute coherence.
-
     nfft : int
       The number of points to use in the fft.
-
     window : string, np.ndarray (default 'hann')
       The window to use for ffts.
-
     debias : bool (default: True)
       Specify whether to debias the signal according to Benignus1969.
-
     noise : tuple(2), or float
       The `noise` keyword may be used to specify the signals'
       noise levels (std of noise in a,b). If `noise` is a two
@@ -122,14 +100,18 @@ def cohere(a, b, nfft, window='hann', debias=True, noise=(0, 0)):
       the noise levels of a and b, respectively.
       default: noise=(0,0)
 
+    Returns
+    -------
+    out : |np.ndarray|
+      Coherence between `a` and `b`
+
     Notes
     -----
-
     Coherence is defined as:
 
     .. math::
 
-      C_{ab} = \frac{|S_{ab}|^2}{S_{aa} * S_bb}
+      C_{ab} = \\frac{|S_{ab}|^2}{S_{aa} * S_{bb}}
 
     Here :math:`S_{ab}`, :math:`S_{aa}` and :math:`S_{bb}` are the cross,
     and auto spectral densities of the signal `a` and `b`.
@@ -142,8 +124,8 @@ def cohere(a, b, nfft, window='hann', debias=True, noise=(0, 0)):
     elif l[0] > l[1]:
         a, b = b, a
         l = l[::-1]
-    step1, nens, nfft = _stepsize(l[0], nfft)
-    step2, nens, nfft = _stepsize(l[1], nfft, nens=nens)
+    step1, nens, nfft = stepsize(l[0], nfft)
+    step2, nens, nfft = stepsize(l[1], nfft, nens=nens)
     if noise.__class__ not in [list, tuple, np.ndarray]:
         noise = [noise, noise]
     elif len(noise) == 1:
@@ -165,23 +147,18 @@ def cohere(a, b, nfft, window='hann', debias=True, noise=(0, 0)):
 
 def cpsd_quasisync(a, b, nfft, fs, window='hann'):
     """
-    Compute the cross power spectral density (CPSD) of the signals *a* and *b*.
+    Compute the cross power spectral density (CPSD) of the signals `a` and `b`.
 
     Parameters
     ----------
-
     a : |np.ndarray|
       The first signal.
-
     b : |np.ndarray|
       The second signal.
-
     nfft : int
       The number of points in the fft.
-
     fs : float
       The sample rate (e.g. sample/second).
-
     window : {None, 1, 'hann', |np.ndarray|}
       The window to use (default: 'hann'). Valid entries are:
       - None,1               : uses a 'boxcar' or ones window.
@@ -190,21 +167,17 @@ def cpsd_quasisync(a, b, nfft, fs, window='hann'):
 
     Returns
     -------
-
-    cpsd : ndarray
+    cpsd : |np.ndarray|
       The cross-spectral density of `a` and `b`.
-
     See also
     ---------
     :func:`psd`,
-    :func:`cohere`,
+    :func:`coherence`,
     :func:`cpsd`,
     numpy.fft
 
-
     Notes
     -----
-
     `a` and `b` do not need to be 'tightly' synchronized, and can even
     be different lengths, but the first- and last-index of both series
     should be synchronized (to whatever degree you want unbiased
@@ -212,7 +185,9 @@ def cpsd_quasisync(a, b, nfft, fs, window='hann'):
 
     This performs:
 
-    fft(a)*conj(fft(b))
+    .. math::
+
+        fft(a)*conj(fft(b))
 
     Note that this is consistent with :func:`numpy.correlate`.
 
@@ -227,7 +202,7 @@ def cpsd_quasisync(a, b, nfft, fs, window='hann'):
 
     """
     if np.iscomplexobj(a) or np.iscomplexobj(b):
-        raise Exception
+        raise Exception("Velocity cannot be complex")
     l = [len(a), len(b)]
     if l[0] == l[1]:
         return cpsd(a, b, nfft, fs, window=window)
@@ -235,15 +210,14 @@ def cpsd_quasisync(a, b, nfft, fs, window='hann'):
         a, b = b, a
         l = l[::-1]
     step = [0, 0]
-    step[0], nens, nfft = _stepsize(l[0], nfft)
-    step[1], nens, nfft = _stepsize(l[1], nfft, nens=nens)
+    step[0], nens, nfft = stepsize(l[0], nfft)
+    step[1], nens, nfft = stepsize(l[1], nfft, nens=nens)
     fs = np.float64(fs)
     window = _getwindow(window, nfft)
     fft_inds = slice(1, int(nfft / 2. + 1))
     wght = 2. / (window ** 2).sum()
     pwr = fft(detrend(a[0:nfft]) * window)[fft_inds] * \
         np.conj(fft(detrend(b[0:nfft]) * window)[fft_inds])
-    print(pwr.dtype)
     if nens - 1:
         for i1, i2 in zip(range(step[0], l[0] - nfft + 1, step[0]),
                           range(step[1], l[1] - nfft + 1, step[1])):
@@ -255,32 +229,23 @@ def cpsd_quasisync(a, b, nfft, fs, window='hann'):
 
 def cpsd(a, b, nfft, fs, window='hann', step=None):
     """
-    Compute the cross power spectral density (CPSD) of the signals *a* and *b*.
-
-    This performs:
-    fft(a)*conj(fft(b))
+    Compute the cross power spectral density (CPSD) of the signals `a` and `b`.
 
     Parameters
     ----------
-
     a : |np.ndarray|
       The first signal.
-
     b : |np.ndarray|
       The second signal.
-
     nfft : int
       The number of points in the fft.
-
     fs : float
       The sample rate (e.g. sample/second).
-
     window : {None, 1, 'hann', |np.ndarray|}
       The window to use (default: 'hann'). Valid entries are:
       - None,1               : uses a 'boxcar' or ones window.
       - 'hann'               : hanning window.
       - a length(nfft) array : use this as the window directly.
-
     step : int
       Use this to specify the overlap.  For example:
       - step : nfft/2 specifies a 50% overlap.
@@ -289,43 +254,44 @@ def cpsd(a, b, nfft, fs, window='hann', step=None):
       By default, `step` is calculated to maximize data use, have
       at least 50% overlap and minimize the number of ensembles.
 
-
     Returns
     -------
-
-    cpsd : ndarray
+    cpsd : |np.ndarray|
       The cross-spectral density of `a` and `b`.
 
-
     See also
     --------
-
     :func:`psd`
-    :func:`cohere`
-    numpy.fft
-    :func:`cpsd_quasisync`
+    :func:`coherence`
+    `numpy.fft`
 
     Notes
     -----
-
     cpsd removes a linear trend from the signals.
 
     The two signals should be the same length, and should both be real.
 
+    This performs:
+
+    .. math::
+
+        fft(a)*conj(fft(b))
+
     This implementation is consistent with the numpy.correlate
     definition of correlation.  (The conjugate of D.B. Chelton's
     definition of correlation.)
 
     The units of the spectra is the product of the units of `a` and
     `b`, divided by the units of fs.
+
     """
     if np.iscomplexobj(a) or np.iscomplexobj(b):
-        raise Exception
+        raise Exception("Velocity cannot be complex")
     auto_psd = False
     if a is b:
         auto_psd = True
     l = len(a)
-    step, nens, nfft = _stepsize(l, nfft, step=step)
+    step, nens, nfft = stepsize(l, nfft, step=step)
     fs = np.float64(fs)
     window = _getwindow(window, nfft)
     fft_inds = slice(1, int(nfft / 2. + 1))
@@ -337,15 +303,13 @@ def cpsd(a, b, nfft, fs, window='hann', step=None):
         pwr = s1 * np.conj(fft(detrend(b[0:nfft]) * window)[fft_inds])
     if nens - 1:
         for i in range(step, l - nfft + 1, step):
-            # print( (i) )
             s1 = fft(detrend(a[i:(i + nfft)]) * window)[fft_inds]
             if auto_psd:
                 pwr += np.abs(s1) ** 2
             else:
-                pwr += s1 * np.conj(fft(detrend(b[i:(i + nfft)]) * window)[fft_inds])
+                pwr += s1 * \
+                    np.conj(fft(detrend(b[i:(i + nfft)]) * window)[fft_inds])
     pwr *= wght / nens / fs
-    # print( 1,step,nens,l,nfft,wght,fs )
-    # error
     return pwr
 
 
@@ -358,32 +322,42 @@ def psd(a, nfft, fs, window='hann', step=None):
     The units of the spectra is the product of the units of `a` and
     `b`, divided by the units of fs.
 
-
     Parameters
     ----------
-    a      : 1d-array_like, the signal. Currently only supports vectors.
-    nfft   : The number of points in the fft.
-    fs     : The sample rate (e.g. sample/second).
-    window : The window to use (default: 'hann'). Valid entries are:
-                 None,1               : uses a 'boxcar' or ones window.
-                 'hann'               : hanning window.
-                 a length(nfft) array : use this as the window directly.
-    step   : Use this to specify the overlap.  For example:
-             -  step=nfft/2 specifies a 50% overlap.
-             -  step=nfft specifies no overlap.
-             -  step=2*nfft means that half the data will be skipped.
-
-             By default, `step` is calculated to maximize data use, have
-             at least 50% overlap and minimize the number of ensembles.
+    a : |np.ndarray|
+      The first signal, as a 1D vector
+    nfft : int
+      The number of points in the fft.
+    fs : float
+      The sample rate (e.g. sample/second).
+    window : {None, 1, 'hann', |np.ndarray|}
+      The window to use (default: 'hann'). Valid entries are:
+      - None,1               : uses a 'boxcar' or ones window.
+      - 'hann'               : hanning window.
+      - a length(nfft) array : use this as the window directly.
+    step : int
+      Use this to specify the overlap.  For example:
+      - step : nfft/2 specifies a 50% overlap.
+      - step : nfft specifies no overlap.
+      - step=2*nfft means that half the data will be skipped.
+      By default, `step` is calculated to maximize data use, have
+      at least 50% overlap and minimize the number of ensembles.
 
     Returns
     -------
-    pow    : 1d-array
+    cpsd : |np.ndarray|
+      The cross-spectral density of `a` and `b`.
 
-    --credit: This was copied from JN's fast_psd.m routine. --
+    Notes
+    -----
+    Credit: This function's line of code was copied from JN's fast_psd.m 
+    routine.
 
-    See also:
-    numpy.fft
+    See Also
+    --------
+    :func:`cpsd`
+    :func:`coherence`
+    `numpy.fft`
 
     """
     return np.abs(cpsd(a, a, nfft, fs, window=window, step=step))
@@ -391,8 +365,8 @@ def psd(a, nfft, fs, window='hann', step=None):
 
 def phase_angle(a, b, nfft, window='hann', step=None):
     """
-    Compute the phase difference between signals a and b. This is the
-    complimentary function to cohere and cpsd.
+    Compute the phase difference between signals `a` and `b`. This is the
+    complimentary function to coherence and cpsd.
 
     Positive angles means that `b` leads `a`, i.e. this does,
     essentially:
@@ -424,15 +398,15 @@ def phase_angle(a, b, nfft, window='hann', step=None):
 
     Returns
     -------
-    ang    : complex numpy_array (unit magnitude values)
+    ang    : complex |np.ndarray| (unit magnitude values)
 
-    See also:
-    numpy.fft
-    cohere
-    cpsd
+    See Also
+    --------
+    `numpy.fft`
+    :func:`coherence`
+    :func:`cpsd`
 
     """
-
     window = _getwindow(window, nfft)
     fft_inds = slice(1, int(nfft / 2. + 1))
     s1 = fft(detrend(a[0:nfft]) * window)[fft_inds]
@@ -441,10 +415,9 @@ def phase_angle(a, b, nfft, window='hann', step=None):
     s2 /= np.abs(s2)
     ang = s2 / s1
     l = len(a)
-    step, nens, nfft = _stepsize(l, nfft, step=step)
+    step, nens, nfft = stepsize(l, nfft, step=step)
     if nens - 1:
         for i in range(step, l - nfft + 1, step):
-            # print( (i) )
             s1 = fft(detrend(a[i:(i + nfft)]) * window)[fft_inds]
             s1 /= np.abs(s1)
             s2 = fft(detrend(a[i:(i + nfft)]) * window)[fft_inds]
